/*
    Copyright (C) 2004-2009, Wolfgang Hottgenroth

    This file is part of smmapdfw.

    smmapdfw is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.  

    smmapdfw is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
    or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details. 

    You should have received a copy of the GNU General Public License
    along with smmapdfw. If not, write to the Free Software Foundation, 59
    Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <syslog.h>
#include <string.h>

#include "containers_public.h"
#include "smmapd.h"

#include "htmalloc.h"
#include "htbuffer.h"
#include "stats.h"

#include "rateconn_worker.h"



counterDef_t *rateconn_counterDefs = NULL;
void rateconn_setupCounterList() {
  rateconn_counterDefs = addOneCounterDef(rateconn_counterDefs, SMM_TYPE_COUNTER, "RateConn: checker");
  rateconn_counterDefs = addOneCounterDef(rateconn_counterDefs, SMM_TYPE_COUNTER, "RateConn: illegal input");
  rateconn_counterDefs = addOneCounterDef(rateconn_counterDefs, SMM_TYPE_COUNTER, "RateConn: returned OK");
  rateconn_counterDefs = addOneCounterDef(rateconn_counterDefs, SMM_TYPE_COUNTER, "RateConn: returned NOK");
  rateconn_counterDefs = addOneCounterDef(rateconn_counterDefs, SMM_TYPE_COUNTER, "RateConn: returned TNOK");
};

#define STAT_CNT_RATECONN_WORKER 0
#define STAT_CNT_RATECONN_ILLEGAL_INPUT 1
#define STAT_CNT_RATECONN_RETURNED_OK 3
#define STAT_CNT_RATECONN_RETURNED_NOK 4
#define STAT_CNT_RATECONN_RETURNED_TNOK 5


struct rateconn_container_handle_s {
  cfgl_t *cfg;
  statCounter_t *statCounter;
};

typedef struct rateconn_container_handle_s rateconn_container_handle_t;



struct rateconn_work_handle_s {
  int id;
  rateconn_container_handle_t *cch;
};

typedef struct rateconn_work_handle_s rateconn_work_handle_t;



int rateconn_init(cfgl_t *cfg, void **handle);
int rateconn_destroy(void *handle);
/* int rateconn_work_setup(void *handle, void **work_handle);  */
int rateconn_work(void *handle, void *work_handle, char *input, htbuffer_t *output);
/* int rateconn_work_destroy(void *handle, void *work_handle);  */


class_descriptor_t rateconn_check = {
  "rateconn_check",
  &rateconn_init,
  &rateconn_destroy,
  NULL,  /* &rateconn_work_setup, */
  &rateconn_work,
  NULL   /* &rateconn_work_destroy */
};


int rateconn_init(cfgl_t *cfg, void **handle) {
  rateconn_container_handle_t *cch;

  cch = (rateconn_container_handle_t*) htmalloc(sizeof(rateconn_container_handle_t));
  cch->cfg = cfg;

  rateconn_setupCounterList();
  cch->statCounter = initStatCounter("rateconn", rateconn_counterDefs);
					    
  *handle = cch;
  return 0;
}

int rateconn_destroy(void *handle) {
  rateconn_container_handle_t *cch = (rateconn_container_handle_t*)handle;

  free(cch);
  return 0;
}




int rateconn_work(void *handle, void *work_handle, char *input, htbuffer_t *output) {
  static const char *DEFAULT_ANSWER = "default answer";
  static const char *ILLEGAL_INPUT = "illegal input (must be 'auth_author')";
  static const char *GO_AHEAD = "go ahead";

  static int log_id = 0;

  rateconn_container_handle_t *cch = (rateconn_container_handle_t*) handle;
  int result = SMM_TEMP_NOK;
  char *auth_author, *response_text, *tmp_arg;

  log_id++;

  syslog(LOG_INFO, "rateconn_work: (%04x) going to check %s", log_id, input);

  incStatCounter(cch->statCounter, STAT_CNT_RATECONN_WORKER);


  result = rateconnCheckWrapper(input);


  

  switch (result) {
  case SMM_LOCAL_NOK:
    htbuffer_strcat(output, "<NOK><REJECTED>");
    result = SMM_OK;
    incStatCounter(cch->statCounter, STAT_CNT_RATECONN_RETURNED_NOK);
    break;
  case SMM_LOCAL_OK:
    htbuffer_strcat(output, "<OK><ACCEPTED>");
    result = SMM_OK;
    incStatCounter(cch->statCounter, STAT_CNT_RATECONN_RETURNED_OK);
    break;
  default:
    htbuffer_strcat(output, "<TNOK><UNKNOWN FAILURE>");
    result = SMM_TEMP_NOK;
    incStatCounter(cch->statCounter, STAT_CNT_RATECONN_RETURNED_TNOK);
    break;
  }


  syslog(LOG_DEBUG, "rateconn_work: (%04x) result %d, %s", log_id,
	 result, output->buf);

  return result;
}

